#include "tester.h"
#include "aliases.h"
#include "termcolour/termcolour.h"
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

internal void print_test_result(TestFuncResult result);

void run_tests(TestFunc *func1, ...) {
  printf("\n");

  print_test_result(func1());

  va_list args;
  va_start(args, func1);

  TestFunc *func = NULL;

  while ((func = va_arg(args, TestFunc *))) {
    TestFuncResult result = func();
    print_test_result(result);

    if (!result.passed) {
      exit(EXIT_FAILURE);
    }
  }

  va_end(args);

  printf("\n");
}

internal void print_test_result(TestFuncResult result) {
  const char *colour;
  const char *result_text;

  if (result.passed) {
    colour = TERM_COLOUR_FG_BR_GREEN;
    result_text = "PASSED";
  } else {
    colour = TERM_COLOUR_FG_BR_RED;
    result_text = "FAILED";
  }

  printf("[%s%s%s%s] %s\n", colour, TERM_COLOUR_BOLD, result_text,
         TERM_COLOUR_CLEAR, result.name);
}
